import React, { useState, useEffect } from 'react';
import {
    Table,
    TableBody,
    TableCell,
    TableContainer,
    TableHead,
    TableRow,
    Paper,
    Tooltip,
    TextField,
    Typography,
    Button
} from '@mui/material';
import { SpreadsheetService } from '../../openapi';

const SpreadsheetTable = ({ raw_data, errors, headers, setRawData }) => {
    const [localErrors, setLocalErrors] = useState(errors || []);
    const [editingCell, setEditingCell] = useState({});

    const generateErrorMap = (errorsList) => {
        const errorMap = new Map();
        if (Array.isArray(errorsList)) {
            errorsList.forEach(error => {
                const key = `${error.row}-${headers[error.cell]}`;
                errorMap.set(key, error.message);
            });
        }
        return errorMap;
    };

    const errorMap = generateErrorMap(localErrors);

    const handleCellEdit = async (rowIndex, colIndex) => {
        const updatedRawData = [...raw_data];
        const columnName = headers[colIndex];
        const currentRow = updatedRawData[rowIndex];
        const newValue = editingCell[`${rowIndex}-${colIndex}`];

        if (newValue === undefined) return;

        console.log(`Editing cell at row ${rowIndex}, column ${colIndex}: new value: ${newValue}`);

        // Ensure currentRow.data exists before accessing it
        if (!currentRow.data) {
            currentRow.data = [];
        }

        // Update the relevant cell value
        currentRow.data[colIndex] = newValue;

        setEditingCell(prev => {
            const updated = { ...prev };
            delete updated[`${rowIndex}-${colIndex}`];
            return updated;
        });

        try {
            const response = await SpreadsheetService.validateCellValidateCellPost({
                row: currentRow.row_num,
                column: columnName,
                value: newValue
            });

            console.log('Validation response:', response);

            if (response.is_valid !== undefined) {
                if (response.is_valid) {
                    // Remove error if it passes validation
                    const updatedErrors = localErrors.filter(
                        error => !(error.row === currentRow.row_num && error.cell === colIndex)
                    );
                    setLocalErrors(updatedErrors);
                } else {
                    const updatedErrors = [
                        ...localErrors,
                        { row: currentRow.row_num, cell: colIndex, message: response.message || 'Invalid value.' }
                    ];
                    setLocalErrors(updatedErrors);
                }
            } else {
                console.error('Unexpected response structure:', response);
            }
            setRawData(updatedRawData);
        } catch (error) {
            console.error('Validation failed:', error);
        }
    };

    const handleCellBlur = (rowIndex, colIndex) => {
        handleCellEdit(rowIndex, colIndex);
    };

    const validateBeforeSubmit = () => {
        let isValid = true;
        raw_data.forEach((row, rowIndex) => {
            headers.forEach((header, colIndex) => {
                const key = `${row.row_num}-${header}`;
                const newValue = editingCell[`${rowIndex}-${colIndex}`];

                if (newValue !== undefined) {
                    // Perform cell validation on each edit
                    handleCellEdit(rowIndex, colIndex);
                }

                const errorMessage = errorMap.get(key);
                if (errorMessage) {
                    isValid = false;
                }
            });
        });
        return isValid;
    };

    const handleSubmit = async () => {
        const isValid = validateBeforeSubmit();

        if (isValid) {
            console.log('All data is valid. Proceeding with submission...');
            // Use validated data to populate the database
            // You can call a dedicated API endpoint to process the entire dataset as needed
        } else {
            console.log('There are validation errors in the dataset. Please correct them before submission.');
        }
    };

    useEffect(() => {
        console.log('Raw data:', raw_data);
        console.log('Errors:', localErrors);
        console.log('Headers:', headers);
    }, [raw_data, localErrors, headers]);

    if (!raw_data || !headers) {
        return <div>Loading...</div>;
    }

    return (
        <TableContainer component={Paper}>
            <Table>
                <TableHead>
                    <TableRow>
                        {headers.map((header, index) => (
                            <TableCell key={index} align="center">
                                <Typography variant="body2">{header}</Typography>
                            </TableCell>
                        ))}
                    </TableRow>
                </TableHead>
                <TableBody>
                    {raw_data.map((row, rowIndex) => (
                        <TableRow key={rowIndex}>
                            {headers.map((header, colIndex) => {
                                const key = `${row.row_num}-${header}`;
                                const errorMessage = errorMap.get(key);
                                const isInvalid = !!errorMessage;
                                const cellValue = (row.data && row.data[colIndex]) || "";
                                const editingValue = editingCell[`${rowIndex}-${colIndex}`];

                                return (
                                    <TableCell key={colIndex} align="center">
                                        <Tooltip title={errorMessage || ""} arrow disableHoverListener={!isInvalid}>
                                            <TextField
                                                value={editingValue !== undefined ? editingValue : cellValue}
                                                onChange={(e) => setEditingCell({ ...editingCell, [`${rowIndex}-${colIndex}`]: e.target.value })}
                                                onKeyDown={(e) => {
                                                    if (e.key === "Enter") {
                                                        handleCellEdit(rowIndex, colIndex);
                                                    }
                                                }}
                                                onBlur={() => handleCellBlur(rowIndex, colIndex)}
                                                error={isInvalid}
                                                fullWidth
                                                variant="outlined"
                                                size="small"
                                            />
                                        </Tooltip>
                                    </TableCell>
                                );
                            })}
                        </TableRow>
                    ))}
                </TableBody>
            </Table>
            <Button variant="contained" color="primary" onClick={handleSubmit}>
                Submit
            </Button>
        </TableContainer>
    );
};

export default SpreadsheetTable;