import React, { useState, useEffect } from 'react';
import {
    Dialog,
    DialogTitle,
    DialogContent,
    DialogActions,
    Button,
    Typography,
    IconButton,
    Box
} from '@mui/material';
import CloseIcon from '@mui/icons-material/Close';
import DownloadIcon from '@mui/icons-material/Download';
import UploadFileIcon from '@mui/icons-material/UploadFile';
import logo from '../assets/Heidi-logo.png';
import { OpenAPI, SpreadsheetService } from '../../openapi';
import type { Body_upload_file_upload_post } from '../../openapi/models/Body_upload_file_upload_post';
import SpreadsheetTable from './SpreadsheetTable';
import Modal from './Modal';  // Import the custom Modal component

interface UploadDialogProps {
    open: boolean;
    onClose: () => void;
}

const UploadDialog: React.FC<UploadDialogProps> = ({ open, onClose }) => {
    const [uploadError, setUploadError] = useState<string | null>(null);
    const [fileSummary, setFileSummary] = useState<{
        data: any[];
        errors: { row: number, cell: number, value: any, message: string }[];
        raw_data: { row_num: number, data: any[] }[];
        dewars_count: number;
        dewars: string[];
        pucks_count: number;
        pucks: string[];
        samples_count: number;
        samples: string[];
        headers: string[]; // Headers must be part of this object
    } | null>(null);
    const [isModalOpen, setIsModalOpen] = useState(false);

    useEffect(() => {
        OpenAPI.BASE = 'http://127.0.0.1:8000';
    }, []);

    const handleFileUpload = async (event: React.ChangeEvent<HTMLInputElement>) => {
        const file = event.target.files?.[0];
        if (!file) return;

        setUploadError(null);
        setFileSummary(null);

        if (!file.name.endsWith('.xlsx')) {
            setUploadError('Invalid file format. Please upload an .xlsx file.');
            return;
        }

        const formData: Body_upload_file_upload_post = {
            file: file,
        } as Body_upload_file_upload_post;

        try {
            const response = await SpreadsheetService.uploadFileUploadPost(formData);

            // Ensure headers are included in the response
            const { headers, raw_data, errors } = response;
            setFileSummary({
                data: raw_data,
                errors: errors,
                raw_data: raw_data,  // keep raw data as is
                headers: headers,  // Set headers correctly here
                dewars_count: 2,  // You might need to adjust according to actual response
                dewars: ['Dewar1', 'Dewar2'],
                pucks_count: 2,  // Adjust accordingly
                pucks: ['Puck1', 'Puck2'],
                samples_count: 23,  // Adjust accordingly
                samples: ['Sample1', 'Sample2']
            });
            setIsModalOpen(true); // Open modal once data is available
        } catch (error) {
            setUploadError('Failed to upload file. Please try again.');
        }
    };

    const handleRawDataChange = (updatedRawData) => {
        setFileSummary((prevSummary) => ({
            ...prevSummary,
            raw_data: updatedRawData
        }));
    };

    return (
        <>
            <Dialog open={open} onClose={onClose} fullWidth maxWidth="sm">
                <DialogTitle>
                    <Box display="flex" justifyContent="space-between" alignItems="center">
                        <Typography variant="h6">Upload Sample Data Sheet</Typography>
                        <IconButton onClick={onClose}>
                            <CloseIcon />
                        </IconButton>
                    </Box>
                </DialogTitle>
                <DialogContent dividers>
                    <Box display="flex" flexDirection="column" alignItems="center" mb={2}>
                        <img src={logo} alt="Logo" style={{ width: 200, marginBottom: 16 }} />
                        <Typography variant="subtitle1">Latest Spreadsheet Template Version 7</Typography>
                        <Typography variant="body2" color="textSecondary">Last update: November 7, 2024</Typography>
                        <Button variant="outlined" startIcon={<DownloadIcon />} href="http://127.0.0.1:8000/download-template" download sx={{ mt: 1 }}>
                            Download XLSX
                        </Button>
                        <Typography variant="subtitle1" sx={{ mt: 3 }}>Latest Spreadsheet Instructions Version 2.3</Typography>
                        <Typography variant="body2" color="textSecondary">Last updated: October 18, 2024</Typography>
                        <Button variant="outlined" startIcon={<DownloadIcon />} href="/path/to/instructions.pdf" download sx={{ mt: 1 }}>
                            Download PDF
                        </Button>
                    </Box>
                    <Box mt={3}>
                        <Button variant="contained" component="label" startIcon={<UploadFileIcon />}>
                            Choose a File
                            <input type="file" hidden onChange={handleFileUpload} />
                        </Button>
                        {uploadError && <Typography color="error">{uploadError}</Typography>}
                    </Box>
                </DialogContent>
                <DialogActions>
                    <Button onClick={onClose} color="primary">Close</Button>
                </DialogActions>
            </Dialog>

            {/* Open modal to display spreadsheet table if file uploaded successfully */}
            {fileSummary && (
                <Modal open={isModalOpen} onClose={() => setIsModalOpen(false)} title="File Summary">
                    <Typography color="success.main">File uploaded successfully!</Typography>
                    <SpreadsheetTable
                        raw_data={fileSummary.raw_data}
                        errors={fileSummary.errors}
                        headers={fileSummary.headers} // Ensure headers are passed here
                        setRawData={handleRawDataChange}
                    />
                    <Typography>Dewars: {fileSummary.dewars_count}</Typography>
                    <Typography>Pucks: {fileSummary.pucks_count}</Typography>
                    <Typography>Samples: {fileSummary.samples_count}</Typography>
                </Modal>
            )}
        </>
    );
};

export default UploadDialog;
